/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.vci3.VciDeviceInfo;

/**
 * The VCI device enumerator manages the list of installed VCI device objects.
 * Use the {@link de.ixxat.vci3.IVciDeviceManager#EnumDevices()} method to get a VCI device
 * enumerator object. When no longer needed the VCI device enumerator object has
 * to be disposed using the {@link IDisposable} interface.
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface IVciEnumDevice extends IDisposable
{
  /**
   * Predefined timeout interval for {@link #WaitFor(int)}.
   * 
   * @see #WaitFor(int)
   */
  public static final int INFINITE = -1;

  /**
   * Returns the info to the next VCI device in the list and increments the
   * internal counter to return the next VCI device with the next call.
   * 
   * @return {@link VciDeviceInfo} - The information of the VCI device.
   * @throws Throwable
   */
  public VciDeviceInfo Next() throws Throwable;

  
  /**
   * Skips the given number of VCI devices by incrementing the internal counter
   * by the value given.
   * 
   * @param dwNumElements Number of VCI devices to skip.
   * @throws Throwable
   */
  public void Skip(int dwNumElements) throws Throwable;
  
  /**
   * Resets the internal counter so that with the next call of {@link #Next()}
   * the information of the first VCI device is returned.
   * 
   * @throws Throwable
   */
  public void Reset() throws Throwable;
  

  /**
   * The WaitFor function returns when a change in the VCI Device list was
   * detected. The change detection starts from new when the Device Enumerator
   * Object is created or the {@link #WaitFor(int)} returns signaling the
   * changed list. If the time-out value has elapsed or the waiting was canceled
   * by a call to {@link #CancelWait()} the function returns with an exception.
   * 
   * @param dwMilliseconds
   *          Time-out interval, in milliseconds. The function returns if the
   *          interval elapses, even if no message was received. If
   *          dwMilliseconds is {@link #INFINITE}, the function's time-out
   *          interval never elapses.
   * @throws Throwable
   */
  public void WaitFor(int dwMilliseconds) throws Throwable;

  /**
   * Unblocks a thread waiting in {@link #WaitFor(int)}. If any thread was
   * waiting, the waiting thread will return with an exception.
   * 
   * @see #WaitFor(int)
   * 
   * @throws Throwable
   */
  public void CancelWait() throws Throwable;

}
