/**
 * 
 */
package de.ixxat.vci3.bal;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;

/**
 * This class implements the base functionality of a transmit FIFO.
 * 
 * @author Michael Ummenhofer
 *
 */
public class FifoWriter extends Fifo
{
  private static final int PutDataEntry   = 12;
  private static final int AcquireWrite   = 13;
  private static final int ReleaseWrite   = 14;

  // Interface functions
  private CFunction m_fpPutDataEntry    = null;
  private CFunction m_fpAcquireWrite    = null;
  private CFunction m_fpReleaseWrite    = null;

  /**
   * constructor
   * 
   * @param pIFifoWriter
   * @throws Throwable
   */
  protected FifoWriter(CMalloc pIFifoWriter) throws Throwable
  {
    super(pIFifoWriter, "IFifoWriter", 1);
    
    try
    {
      // Retrieve remaining Interface Methods
      m_fpPutDataEntry    = new CFunction(m_IFifo, PutDataEntry,   "JNI");
      m_fpAcquireWrite    = new CFunction(m_IFifo, AcquireWrite,   "JNI");
      m_fpReleaseWrite    = new CFunction(m_IFifo, ReleaseWrite,   "JNI");
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_fpPutDataEntry    = null;
      m_fpAcquireWrite    = null;
      m_fpReleaseWrite    = null;

      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_fpPutDataEntry    = null;
      m_fpAcquireWrite    = null;
      m_fpReleaseWrite    = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * Acquires a {@link de.ixxat.jni.CPointer} which represents a FIFO entry.
   * Call {@link #ReleaseWrite()} to release the FIFO entry.
   * 
   * @return CPointer - The C- Pointer to the FIFO entry.
   * @throws Throwable
   */
  protected CPointer AcquireWrite() throws Throwable
  {
    int       hResult = 0;
    CPointer  pData   = null;
    short     wCount  = 0; 

    try
    {
      // Get Message Pointer
      hResult = m_fpAcquireWrite.callInt(new Object[]{ m_IFifo, m_ppvData, m_pwCount});
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->AcquireWrite failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        pData  = m_ppvData.getCPointer(0);
        wCount = m_pwCount.getShort(0);
        if(wCount == 0)
        {
          throw new VciException(m_strInstanceName + "->AcquireWrite failed, no more free entries available.", hResult);
        }
      }
    }
    finally
    {
    }

    return pData;
  }
  
  
  /**
   * Releases one FIFO entry, acquired with {@link #AcquireWrite()} before.
   * 
   * @throws Throwable
   */
  protected void ReleaseWrite() throws Throwable
  {
    int hResult = 0;

    try
    {
      // Release Message
      hResult = m_fpReleaseWrite.callInt(new Object[]{ m_IFifo, new Integer(1) });
      if(hResult != 0)
      {
        throw new VciException(m_strInstanceName + "->ReleaseWrite failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }
}
