/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.vci3.IDisposable;

/**
 * This interface represents a CAN message writer. When no longer needed the CAN
 * communication channel object has to be disposed using the {@link IDisposable}
 * interface. It's used to write CAN messages to a CAN communication channel for
 * transmission (see {@link ICanChannel}). A CAN message writer object can be
 * obtained via the method {@link ICanChannel#GetMessageWriter()}.
 * 
 * @author Michael Ummenhofer
 */
public interface ICanMessageWriter extends IDisposable
{
  /**
   * Predefined timeout interval for {@link #WaitFor(int)}.
   * 
   * @see #WaitFor(int)
   */
  public static final int INFINITE = -1;

  /**
   * Gets the capacity of the transmit FIFO in number of CAN messages.
   * 
   * @return Capacity
   * @throws Throwable
   */
  public int GetCapacity() throws Throwable;

  /**
   * Gets the number of currently unsent CAN messages within the transmit FIFO.
   * 
   * @return FillCount
   * @throws Throwable
   */
  public int GetFillCount() throws Throwable;

  /**
   * Gets the number of currently free CAN messages within the transmit FIFO.
   * 
   * @return FreeCount
   * @throws Throwable
   */
  public int GetFreeCount() throws Throwable;

  /**
   * The WaitFor function returns when a new CAN message can be written to the
   * FIFO or the time-out interval elapses. If the time-out value has elapsed or
   * the waiting was canceled by a call to {@link #CancelWait()} the function
   * returns with an exception.
   * 
   * @param dwMilliseconds
   *          Time-out interval, in milliseconds. The function returns if the
   *          interval elapses, even if no new message can be written in the
   *          FIFO. If dwMilliseconds is {@link #INFINITE}, the function's
   *          time-out interval never elapses.
   * @throws Throwable
   */
  public void WaitFor(int dwMilliseconds) throws Throwable;

  /**
   * Unblocks a thread waiting in {@link #WaitFor(int)}. If any thread was
   * waiting, the waiting thread will return with an exception.
   * 
   * @see #WaitFor(int)
   * 
   * @throws Throwable
   */
  public void CancelWait() throws Throwable;

  /**
   * This method places a single CAN message at the end of the transmit FIFO and
   * returns without waiting for the message to be transmitted.
   * 
   * @param oCanMsg
   *          The CanMessage to send.
   * @throws Throwable
   *           If the transmission fails (e.g. the FIFO is full) this function
   *           throws an exception.
   */
  public void WriteMessage(CanMessage oCanMsg) throws Throwable;

}
